#include "mbed.h"
#include "math.h"

// Distance estimation based on Ultasound sensor: 200ms 

/*
  NUCLEO-F334R8
  PINS
**WEELS SECTION    

    LEFT WHEEL
        leftIN1         -> PC_10
        leftIN2         -> PC_12
        leftWenable     -> PA_13

    RIGHT WHEEL
        rightIN1        -> PC_11
        rightIN2        -> PC_14
        rightWenable    -> PC_15

**ULTASENSOR SECTION
  
    TRIGGER
        ultraTrig       -> PC_8
    ECHO RECEIVER
        ultraEcho       -> PC_6
        
*/




#define VERY_CLOSE_OBST    5
#define NEAR_OBST          20 
 
// ****************************************************
DigitalOut led(LED1);



//**************************************************
//**************  Ultrasound variables  ************ 
//**************************************************

DigitalOut      ultraTrig(PC_8) ;
InterruptIn     ultraEcho(PC_6) ;
Timer           EchoTime ;
Ticker          UltaTickerTrig ;
float           UltraDistance ;
float           UltraEchoTime ;



//**************************************************
//**************  Robot control variables ********** 
//**************************************************
DigitalOut leftIN1(PC_10) ;
DigitalOut leftIN2(PC_12) ;
DigitalOut leftWenable(PC_11);

//**************************************************
DigitalOut rightIN1(PC_3) ;
DigitalOut rightIN2(PC_0) ;
DigitalOut rightWenable(PC_2);
     

//**************************************************
//**************  Ultrasound functions  ************ 
//**************************************************

//**************************************************
void UltraTrigStart() {
//    led=1 ;
    ultraTrig = 1 ;
    wait_us(12) ;
    ultraTrig = 0 ;
//    led=0 ;
}

//**************************************************
void UltraEchoStart() {
    EchoTime.reset() ;
    EchoTime.start() ;
    led=1 ;
}


//**************************************************
void UltraEchoEnd() {
    EchoTime.stop() ;
    UltraEchoTime = EchoTime.read();
    UltraDistance = UltraEchoTime * (340.0*50) ;   // estimation in centimeters
    led=0 ;
}


//**************************************************
//**************  Robot control functions ********** 
//**************************************************


// ****************************************************
void forRightWheel() 
{
rightIN1 = 1 ;    
rightIN2 = 0 ;    
rightWenable = 1 ;
}

// ****************************************************
void bacRightWheel() 
{
rightIN1 = 0 ;    
rightIN2 = 1 ;    
rightWenable = 1 ;
}


// ****************************************************
void stopRightWheel() 
{
rightWenable = 0 ;
}


// ****************************************************
void forLeftWheel() 
{
leftIN1 = 1 ;    
leftIN2 = 0 ;    
leftWenable = 1 ;
}

// ****************************************************
void bacLeftWheel() 
{
leftIN1 = 0 ;    
leftIN2 = 1 ;    
leftWenable = 1 ;
}


// ****************************************************
void stopLeftWheel() 
{
leftWenable = 0 ;
}


// ****************************************************
void rightTurn( void )
{
forLeftWheel() ; 
bacRightWheel() ; 
} 

// ****************************************************
void leftTurn( void )
{
bacLeftWheel() ; 
forRightWheel() ; 
} 

// ****************************************************
void forWord( void )
{
forLeftWheel() ; 
forRightWheel() ; 
} 


// ****************************************************
void backWord( void )
{
bacLeftWheel() ; 
bacRightWheel() ; 
} 

//**************  End of Wheels Control  ************** 


// ****************************************************
// ****************************************************
// ****************************************************
int main() {

//  OK
    UltaTickerTrig.attach(&UltraTrigStart, 0.2 );   // every 200ms
    ultraEcho.rise(&UltraEchoStart) ;
    ultraEcho.fall(&UltraEchoEnd) ;

while(1) 
    printf("D=%6.2f %7.4f s\r\n", 
    UltraDistance, UltraEchoTime  );



    while(1) {
        if (UltraDistance < VERY_CLOSE_OBST ) { 
            backWord() ;
                wait(0.5) ;
            }
        
        if (UltraDistance < NEAR_OBST ) 
            if (rand()%2) { 
                rightTurn() ;
                wait(0.2*(rand()%3)) ;
                }
            else { 
                leftTurn() ;
                wait(0.2*(rand()%3)) ;
                }
        else
            forWord() ;
        }

}


